import os

base_directories = {
                    'input':  r'ncds_groups_17fam_nnn',
                    'output': r'alignments_17fam_nnn',
                    'log':    r'logs',
                    'work':   r'work',
                   }
file_extensions = {
                   'input':  '.fasta',
                   'output': '.afa',
                   'log':    '.log',
                  }
command_template = 'start /WAIT /MIN /BELOWNORMAL muscle.exe ' + \
                   '-in {infile} -out {outfile} -log {logfile}'

def main():
    print 'MUSCLE batch runner'
    print
    for dirpath, dirnames, filenames in os.walk(base_directories['input']):
        ##print dirpath, ':'
        for dirname in dirnames:
            ##print "  Dir:", dirname
            indir = os.path.join(dirpath, dirname)
            outdir = subst_start(base_directories['input'],
                                 base_directories['output'],
                                 indir)
            logdir = subst_start(base_directories['input'],
                                 base_directories['log'],
                                 indir)
            ##print "    Out:", outdir
            ##print "    Log:", logdir
            if not os.path.exists(outdir):
                os.makedirs(outdir)
            if not os.path.exists(logdir):
                os.makedirs(logdir)
        for filename in filenames:
            ##print "  File:", filename
            if not filename.endswith(file_extensions['input']):
                ##print "    Skipped"
                continue
            ##print "    Aligning..."
            seq_name = filename[:-len(file_extensions['input'])]
            indir = dirpath
            outdir = subst_start(base_directories['input'],
                                 base_directories['output'],
                                 indir)
            logdir = subst_start(base_directories['input'],
                                 base_directories['log'],
                                 indir)
            make_alignment(seq_name, indir, outdir, logdir)

def subst_start(old_start, new_start, the_string):
    assert the_string.startswith(old_start)
    return new_start + the_string[len(old_start):]

def make_alignment(seq_name, indir, outdir, logdir):
    workdir = base_directories['work']
    if not os.path.exists(workdir):
        os.makedirs(workdir)
    infile = os.path.join(indir, seq_name + file_extensions['input'])
    temp_outfile = os.path.join(workdir, seq_name + file_extensions['output'])
    temp_logfile = os.path.join(workdir, seq_name + file_extensions['log'])
    final_outfile = os.path.join(outdir, seq_name + file_extensions['output'])
    final_logfile = os.path.join(logdir, seq_name + file_extensions['log'])
    print "Input '{0}':".format(infile),
    if os.path.exists(final_outfile):
        print "already done."
        return
    if not os.path.exists(infile):
        exit("File {0} doesn't exist.".format(infile))
    print "running MUSCLE...",
    os.system(command_template.format(infile=infile,
                                      outfile=temp_outfile,
                                      logfile=temp_logfile))
    log = read_file_lines(temp_logfile)
    if log[-1].startswith('Finished'):
        print 'done'
        os.rename(temp_outfile, final_outfile)
        os.rename(temp_logfile, final_logfile)
    else:
        print 'ERROR'
        exit()

def read_file_lines(file_name):
    try:
        with file(file_name) as infile: result = infile.readlines()
    except IOError as e: exit("IOError: {0}".format(e))
    return result

if __name__ == '__main__':
    main()
